@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2014, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,8 +17,6 @@
  * 02110-1301  USA
  */
 
-#include "stdafx.h"
-
 #include <string>
 
 #include "base/ui_form.h"
@@ -270,8 +268,8 @@ void WBContextUI::reset()
   _wb->request_refresh(RefreshSelection, "", 0);
 
   get_physical_overview()->send_refresh_children(bec::NodeId());
-  
-  _wb->get_component<WBComponentPhysical>()->update_catalog_tree_model();
+
+  _wb->get_model_context()->refill_catalog_tree();
 }
 
 
@@ -454,7 +452,7 @@ void WBContextUI::show_help_index()
   GUILock lock(_wb, _("Starting Doc Lib"), _("The MySQL Doc Library is opening currently, "
     "which should be finished in a moment .\n\nPlease stand by..."));
 
-  _wb->execute_plugin("wb.doclib.open");
+  _wb->execute_plugin("wb.docs.open");
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -896,7 +894,7 @@ bool WBContextUI::get_wb_options_value(const std::string &model, const std::stri
   if (!model.empty())
   {
     grt::DictRef model_options(get_model_options(model));
-    int use_global= model_options.get_int("useglobal", 1);
+    bool use_global = model_options.get_int("useglobal", 1) != 0;
 
     if (key == "useglobal")
     {
@@ -954,14 +952,14 @@ void WBContextUI::set_wb_options_value(const std::string &model, const std::stri
   {
   case grt::DoubleType:
     {
-      grt::DoubleRef v(atof(value.c_str()));
+      grt::DoubleRef v(base::atof<double>(value, 0.0));
       if (!options.has_key(key) || options.get_double(key) != *v)
         options.set(key, v);
       break;
     }
   case grt::IntegerType:
     {
-      grt::IntegerRef v(atoi(value.c_str()));
+      grt::IntegerRef v(base::atoi<int>(value, 0));
       if (!options.has_key(key) || options.get_int(key) != *v)
         options.set(key, v);
       break;
@@ -1122,9 +1120,9 @@ bool WBContextUI::start_plugin_install(const std::string &path)
 
 //--------------------------------------------------------------------------------------------------
 
-static struct RegisterWBContextUINotifDocs
+static struct RegisterNotifDocs_wb_context_ui
 {
-  RegisterWBContextUINotifDocs()
+  RegisterNotifDocs_wb_context_ui()
   {
     base::NotificationCenter::get()->register_notification("GNAppStarted",
                                                            "application",
@@ -1151,6 +1149,6 @@ static struct RegisterWBContextUINotifDocs
                                                            "",
                                                            "");
   }
-} initdocs;
+} initdocs_wb_context_ui;
 
 
