@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2015, Oracle and/or its affiliates. All rights reserved.
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -74,17 +74,17 @@ static std::string showFileDialog(const std::string &type, const std::string &ti
     {
       std::string ext= iter->substr(iter->find('|')+1);
       ext= bec::replace_string(ext, "*.", "");
-      [fileTypes addObject:[NSString stringWithUTF8String:ext.c_str()]];
+      [fileTypes addObject:@(ext.c_str())];
     }
     else
-      [fileTypes addObject:[NSString stringWithUTF8String:iter->c_str()]];
+      [fileTypes addObject:@(iter->c_str())];
   }  
   
   if (type == "open")
   {
     NSOpenPanel *panel= [NSOpenPanel openPanel];
 
-    [panel setTitle: [NSString stringWithUTF8String:title.c_str()]];
+    [panel setTitle: @(title.c_str())];
     [panel setAllowedFileTypes: fileTypes];
     
     if ([panel runModal] == NSFileHandlingPanelOKButton)
@@ -94,7 +94,7 @@ static std::string showFileDialog(const std::string &type, const std::string &ti
   {
     NSSavePanel *panel= [NSSavePanel savePanel];
     
-    [panel setTitle:[NSString stringWithUTF8String:title.c_str()]];
+    [panel setTitle:@(title.c_str())];
     
     [panel setAllowedFileTypes:fileTypes];
 
@@ -177,16 +177,15 @@ static NativeHandle windowOpenPlugin(bec::GRTManager *grtm,
     // determine the path for the plugin bundle by stripping Contents/Framework/dylibname 
     bundlePath= [[[[NSString stringWithCPPString:path] stringByDeletingLastPathComponent] stringByDeletingLastPathComponent] stringByDeletingLastPathComponent];
     
-    NSLog(@"opening plugin bundle %@ ([%s initWithModule:GRTManager:arguments:...])", bundlePath, class_name.c_str());
+    NSLog(@"opening plugin bundle %@ ([%s initWithModule:grtManager:arguments:...])", bundlePath, class_name.c_str());
     
     pluginBundle= [NSBundle bundleWithPath: bundlePath];
     if (!pluginBundle)
     {
       NSLog(@"plugin bundle %@ for plugin %s could not be loaded", bundlePath, path.c_str());
       NSRunAlertPanel(NSLocalizedString(@"Error Opening Plugin", @"plugin open error"),
-                      [NSString stringWithFormat:NSLocalizedString(@"The plugin %s could not be loaded.", @"plugin open error"),
-                       shlib.c_str()],
-                      NSLocalizedString(@"OK", @"dialog window"), nil, nil);
+                      NSLocalizedString(@"The plugin %s could not be loaded.", @"plugin open error"),
+                      nil, nil, nil, shlib.c_str());
       return 0;
     }
     /* debug msg
@@ -195,15 +194,14 @@ static NativeHandle windowOpenPlugin(bec::GRTManager *grtm,
     else
       NSLog(@"plugin bundle is supposed to be already loaded");
         */
-    Class pclass= [pluginBundle classNamed:[NSString stringWithUTF8String:class_name.c_str()]];
+    Class pclass= [pluginBundle classNamed:@(class_name.c_str())];
     if (!pclass)
     {
       NSLog(@"plugin class %s was not found in bundle %@", class_name.c_str(), bundlePath);
       
       NSRunAlertPanel(NSLocalizedString(@"Error Opening Plugin", @"plugin open error"),
-                      [NSString stringWithFormat:@"The plugin %s does not contain the published object %s",
-                       shlib.c_str(), class_name.c_str()],
-                      NSLocalizedString(@"OK", @"dialog window"), nil, nil);
+                      @"The plugin %s does not contain the published object %s",
+                      nil, nil, nil, shlib.c_str(), class_name.c_str());
       return 0;
     }
     
@@ -228,8 +226,6 @@ static NativeHandle windowOpenPlugin(bec::GRTManager *grtm,
           // drop the old plugin->handle mapping
           grtm->get_plugin_manager()->forget_gui_plugin_handle(existingPanel);
           
-          [window activatePanel: existingPanel];
-          
           if ([existingPanel respondsToSelector: @selector(pluginEditor)])
           {
             id editor= [existingPanel pluginEditor];
@@ -253,7 +249,7 @@ static NativeHandle windowOpenPlugin(bec::GRTManager *grtm,
     }
     
     // Instantiate and initialize the plugin.
-    id plugin= [[pclass alloc] initWithModule:ownerModule GRTManager:grtm arguments:args];
+    id plugin = [[pclass alloc] initWithModule: ownerModule grtManager: grtm arguments: args];
       
     //NSLog(@"CREATED PLUGIN %@ %@ %@", plugin, [plugin identifier], [plugin title]);
     
@@ -494,7 +490,7 @@ static bool validate_paste(wb::WBContextUI *wbui)
     BOOL isEditable = [(id)[[NSApp keyWindow] firstResponder] isEditable];
     
     // 2) The pasteboard contains text.
-    NSArray* supportedTypes = [NSArray arrayWithObjects: NSStringPboardType, nil];
+    NSArray* supportedTypes = @[NSStringPboardType];
     NSString *bestType = [[NSPasteboard generalPasteboard] availableTypeFromArray: supportedTypes];
 
     return isEditable && (bestType != nil);
@@ -509,8 +505,8 @@ static bool validate_paste(wb::WBContextUI *wbui)
 
 static bool validate_select_all(wb::WBContextUI *wbui)
 {
-  if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(isEditable)])
-    return [(id)[[NSApp keyWindow] firstResponder] isEditable];
+  if ([[[NSApp keyWindow] firstResponder] respondsToSelector: @selector(isSelectable)])
+    return [(id)[[NSApp keyWindow] firstResponder] isSelectable];
 
   return (wbui->get_active_form());
 }
@@ -627,9 +623,8 @@ static bool validate_toggle_fullscreen(WBMainWindow *mainWindow)
 static bool validate_find_replace()
 {
   id firstResponder = [[NSApp keyWindow] firstResponder];
-  
   if ([firstResponder isKindOfClass: [ScintillaView class]] || [firstResponder isKindOfClass: [NSTextView class]] ||
-      ([firstResponder isKindOfClass: [NSView class]] && [[firstResponder superview] isKindOfClass: [ScintillaView class]]))
+      [firstResponder isKindOfClass: [SCIContentView class]])
     return true;
   return false;
 }
@@ -641,9 +636,11 @@ static void call_find_replace(bool do_replace)
   if (validate_find_replace())
   {
     id firstResponder = [[NSApp keyWindow] firstResponder];
-    if ([firstResponder isKindOfClass: [InnerView class]])
-      firstResponder = [firstResponder superview];
-    
+    if ([firstResponder isKindOfClass: [SCIContentView class]])
+    {
+      while (firstResponder && ![firstResponder isKindOfClass: [ScintillaView class]])
+        firstResponder = [firstResponder superview];
+    }
     if ([firstResponder isKindOfClass: [MFCodeEditor class]])
       [firstResponder showFindPanel: do_replace];
   }
@@ -656,9 +653,11 @@ static void call_find_replace(bool do_replace)
 static void call_find(WBMainWindow *mainWindow)
 {
   id firstResponder = [[NSApp keyWindow] firstResponder];
-  if ([firstResponder isKindOfClass: [InnerView class]])
-    firstResponder = [firstResponder superview];
-
+  if ([firstResponder isKindOfClass: [SCIContentView class]])
+  {
+    while (firstResponder && ![firstResponder isKindOfClass: [ScintillaView class]])
+      firstResponder = [firstResponder superview];
+  }
   if ([firstResponder isKindOfClass: [MFCodeEditor class]])
     [firstResponder showFindPanel: NO];
   else
@@ -698,7 +697,7 @@ static bool validate_undo(WBMainWindow *mainWindow)
     return [firstResponder canUndo];
   else if ([firstResponder isKindOfClass: [NSTextView class]])
     return [[firstResponder undoManager] canUndo];
-  else if ([firstResponder isKindOfClass: [InnerView class]])
+  else if ([firstResponder isKindOfClass: [SCIContentView class]])
     return true;
   else
     if (wbui->get_active_main_form())
@@ -729,7 +728,7 @@ static bool validate_redo(WBMainWindow *mainWindow)
     return [firstResponder canRedo];
   else if ([firstResponder isKindOfClass: [NSTextView class]])
     return [[firstResponder undoManager] canRedo];
-  else if ([firstResponder isKindOfClass: [InnerView class]])
+  else if ([firstResponder isKindOfClass: [SCIContentView class]])
     return true;
   else
     if (wbui->get_active_main_form())
@@ -886,7 +885,7 @@ static NSString *applicationSupportFolder()
   NSArray *res= NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
 
   if ([res count] > 0)
-    return [res objectAtIndex: 0];
+    return res[0];
   return @"/tmp/";
 }
 
@@ -946,7 +945,7 @@ static NSString *applicationSupportFolder()
       }
       
       // we ship 32bit binary python modules, so python needs to be started as 32bit as well
-      putenv((char*)"VERSIONER_PYTHON_PREFER_32_BIT=yes");
+      //putenv((char*)"VERSIONER_PYTHON_PREFER_32_BIT=yes");
     }
 
     _wbui->init(&wbcallbacks, _options);
@@ -971,7 +970,7 @@ static NSString *applicationSupportFolder()
   _options->basedir = [[[NSBundle mainBundle] resourcePath] fileSystemRepresentation];
   _options->struct_search_path = _options->basedir + "/grt";
   _options->plugin_search_path = std::string([[[NSBundle mainBundle] builtInPlugInsPath] fileSystemRepresentation]);
-  _options->module_search_path = std::string([[[NSBundle mainBundle] builtInPlugInsPath] fileSystemRepresentation]);
+  _options->module_search_path = std::string([[[NSBundle mainBundle] builtInPlugInsPath] fileSystemRepresentation]) + ":" + std::string([[[NSBundle mainBundle] resourcePath] fileSystemRepresentation]) + "/plugins";
   _options->library_search_path = std::string([[[NSBundle mainBundle] resourcePath] fileSystemRepresentation]) + "/libraries";
   _options->cdbc_driver_search_path = std::string([[[NSBundle mainBundle] privateFrameworksPath] fileSystemRepresentation]);
   _options->user_data_dir= [[applicationSupportFolder() stringByAppendingString: @"/MySQL/Workbench"] fileSystemRepresentation];
@@ -1002,8 +1001,9 @@ extern "C" {
 static void init_mforms()
 {
   log_debug("Initializing mforms\n");
-  
+
   extern void cf_tabview_init();
+  extern void cf_record_grid_init();
   static BOOL inited= NO;
   
   if (!inited)
@@ -1012,6 +1012,7 @@ static void init_mforms()
   
     mforms_cocoa_init();
     cf_tabview_init();
+    cf_record_grid_init();
   }
 }
 
@@ -1024,7 +1025,7 @@ static void init_mforms()
 }
 #endif
 
-- (id)init
+- (instancetype)init
 {
   self= [super init];
   if (self)
@@ -1079,7 +1080,7 @@ static void init_mforms()
   {
     init_mforms();
     
-    [NSApp setDelegate: self];
+    NSApplication.sharedApplication.delegate = self;
     
     // Setup delegate to log symbolic stack traces on exceptions.
     [[NSExceptionHandler defaultExceptionHandler] setExceptionHandlingMask: NSLogUncaughtExceptionMask | NSLogUncaughtSystemExceptionMask | NSLogUncaughtRuntimeErrorMask | NSLogTopLevelExceptionMask | NSLogOtherExceptionMask];
@@ -1142,9 +1143,8 @@ static void init_mforms()
     }
     catch (const std::exception &exc)
     {
-      NSRunAlertPanel(@"Unhandled Exception",
-                      [NSString stringWithFormat: @"An unhandled exception has occurred: %s", exc.what()],
-                      @"OK", nil, nil);
+      NSRunAlertPanel(@"Unhandled Exception", @"An unhandled exception has occurred: %s",
+                      @"OK", nil, nil, exc.what());
     }
     _showingUnhandledException = NO;
   }
@@ -1155,7 +1155,7 @@ static void init_mforms()
 {
   static NSArray *modes= nil;
   if (!modes) 
-    modes= [[NSArray arrayWithObjects:NSDefaultRunLoopMode, NSModalPanelRunLoopMode, nil] retain];
+    modes= [@[NSDefaultRunLoopMode, NSModalPanelRunLoopMode] retain];
   
   // if we call flush_idle_tasks() directly here, we could get blocked by a plugin with a
   // modal loop. In that case, the timer would not get fired again until the modal loop
