@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,7 +27,7 @@
   WBPluginEditorBase *mPluginEditor;  
 }
 
-- (id)initWithPlugin:(WBPluginEditorBase*)plugin;
-- (WBPluginEditorBase*)pluginEditor;
+- (instancetype)initWithPlugin:(WBPluginEditorBase*)plugin NS_DESIGNATED_INITIALIZER;
+@property (readonly, strong) WBPluginEditorBase *pluginEditor;
 
 @end
