@@ -18,18 +18,11 @@
  */
 
 #pragma once
-#if _WIN32
-#define _WIN_OS 1
-#endif
-#if _WIN64
-#define _WIN_OS 1
-#endif
 
-#ifdef _WIN_OS
 #ifdef _WIN64
-typedef __int64 ssize_t;
+  typedef __int64 ssize_t;
 #else
-typedef int ssize_t;
+  typedef int ssize_t;
 #endif
 
 #define WIN32_LEAN_AND_MEAN
@@ -56,4 +49,3 @@ typedef int ssize_t;
 
 #include <glib/glib.h>
 #include <glib/gstdio.h>
-#endif
\ No newline at end of file
