@@ -32,7 +32,7 @@
 
 @implementation DraggingTableView
 
-- (id)initWithFrame: (NSRect)frame owner: (mforms::ListBox*)theOwner
+- (instancetype)initWithFrame: (NSRect)frame owner: (mforms::ListBox*)theOwner
 {
   self = [super initWithFrame: frame];
   if (self != nil)
@@ -53,7 +53,7 @@ STANDARD_FOCUS_HANDLING(self) // Notify backend when getting first responder sta
 
 @implementation MFListBoxImpl
 
-- (id)initWithObject:(::mforms::ListBox*)aListBox
+- (instancetype)initWithObject:(::mforms::ListBox*)aListBox
 {
   self= [super initWithFrame:NSMakeRect(0, 0, 40, 40)];
   if (self)
@@ -66,10 +66,12 @@ STANDARD_FOCUS_HANDLING(self) // Notify backend when getting first responder sta
 
     NSRect rect;
     rect.origin= NSMakePoint(0, 0);
-    rect.size= [NSScrollView contentSizeForFrameSize: [self frame].size
-                               hasHorizontalScroller: NO
-                                 hasVerticalScroller: YES
-                                          borderType: NSBezelBorder];
+    rect.size= [NSScrollView contentSizeForFrameSize: self.frame.size
+                             horizontalScrollerClass: nil
+                               verticalScrollerClass: [NSScroller class]
+                                          borderType: NSBezelBorder
+                                         controlSize: NSRegularControlSize
+                                       scrollerStyle: NSScrollerStyleOverlay];
     mTable = [[[DraggingTableView alloc] initWithFrame: rect owner: aListBox] autorelease];
     mHeader= [[mTable headerView] retain];
     [mTable setHeaderView: nil];
@@ -122,7 +124,7 @@ STANDARD_FOCUS_HANDLING(self) // Notify backend when getting first responder sta
 
 - (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(NSInteger)rowIndex
 {
-  return [mContents objectAtIndex: rowIndex];
+  return mContents[rowIndex];
 }
 
 
@@ -250,7 +252,7 @@ static std::string listbox_get_text(::mforms::ListBox *self)
     if ( listbox )
     {
       if ([listbox->mTable selectedRow] >= 0)
-        return [[listbox->mContents objectAtIndex: [listbox->mTable selectedRow]] UTF8String];
+        return [listbox->mContents[[listbox->mTable selectedRow]] UTF8String];
     }
   }
   return "";
