@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,21 +24,33 @@
 #import "MFBase.h"
 #include <stdexcept>
 
+#include "base/log.h"
+#include "base/string_utilities.h"
+
+DEFAULT_LOG_DOMAIN("utilities");
+
+static void util_beep()
+{
+  NSBeep();
+}
+
 static int util_show_message(const std::string &title, const std::string &text,
                              const std::string &ok, const std::string &cancel,
                              const std::string &other)
 {
-  int res= NSRunAlertPanel(wrap_nsstring(title), [wrap_nsstring(text) stringByReplacingOccurrencesOfString:@"%" withString:@"%%"], 
-                           ok.empty() ? nil : wrap_nsstring(ok),
-                           other.empty() ? nil : wrap_nsstring(other),
-                           cancel.empty() ? nil : wrap_nsstring(cancel));
-  
+  int res = [[NSAlert alertWithMessageText: wrap_nsstring(title)
+                             defaultButton: ok.empty() ? nil : wrap_nsstring(ok)
+                           alternateButton: cancel.empty() ? nil : wrap_nsstring(cancel)
+                               otherButton: other.empty() ? nil : wrap_nsstring(other)
+                 informativeTextWithFormat: @"%@",
+              [wrap_nsstring(text) stringByReplacingOccurrencesOfString: @"%"
+                                                             withString: @"%%"]] runModal];
   if (res == NSAlertDefaultReturn)
     return mforms::ResultOk;
   else if (res == NSAlertOtherReturn)
-    return mforms::ResultCancel;
-  else
     return mforms::ResultOther;
+  else
+    return mforms::ResultCancel;
 }
 
 
@@ -80,8 +92,8 @@ static int util_show_message_with_checkbox(const std::string &title, const std::
 static void util_set_clipboard_text(const std::string &text)
 {
   NSPasteboard *pasteBoard= [NSPasteboard generalPasteboard];
-  [pasteBoard declareTypes: [NSArray arrayWithObject:NSStringPboardType] owner:nil];
-  [pasteBoard setString: [NSString stringWithUTF8String:text.c_str()]
+  [pasteBoard declareTypes: @[NSStringPboardType] owner:nil];
+  [pasteBoard setString: @(text.c_str())
                                       forType: NSStringPboardType];
 }
 
@@ -94,9 +106,11 @@ static std::string util_get_clipboard_text()
 static void util_open_url(const std::string &url)
 {
   if (g_file_test(url.c_str(), G_FILE_TEST_EXISTS) || (!url.empty() && url[0] == '/'))
-    [[NSWorkspace sharedWorkspace] openURL:[NSURL fileURLWithPath: [NSString stringWithUTF8String: url.c_str()]]];
+    [[NSWorkspace sharedWorkspace] openURL:[NSURL fileURLWithPath: @(url.c_str())]];
   else
-    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString: [NSString stringWithUTF8String: url.c_str()]]];
+    if (![[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString: [@(url.c_str())
+                                                                       stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding]]])
+      log_error("Could not open URL %s\n", url.c_str());
 }
 
 
@@ -125,14 +139,19 @@ static std::string get_special_folder(mforms::FolderType type)
   boost::function<bool ()> *callback;
 }
 
-- (id)initWithSlot:(boost::function<bool ()>)slot;
+- (instancetype)initWithSlot:(boost::function<bool ()>)slot NS_DESIGNATED_INITIALIZER;
 - (void)fire:(NSTimer*)timer;
 
 @end
 
+
+static std::map<mforms::TimeoutHandle, NSTimer*> active_timeouts;
+static mforms::TimeoutHandle current_timeout = 0;
+static base::Mutex timeout_lock;
+
 @implementation MFTimerHandler
 
-- (id)initWithSlot:(boost::function<bool ()>)slot
+- (instancetype)initWithSlot:(boost::function<bool ()>)slot
 {
   self = [super init];
   if (self)
@@ -150,26 +169,64 @@ static std::string get_special_folder(mforms::FolderType type)
 
 - (void)fire:(NSTimer*)timer
 {
-  bool ret = (*callback)();
-  if (!ret)
+  try
   {
-    [timer invalidate];
-    [self autorelease];
+    bool ret = (*callback)();
+    if (!ret)
+    {
+      {
+        base::MutexLock lock(timeout_lock);
+
+        for (std::map<mforms::TimeoutHandle, NSTimer*>::iterator it = active_timeouts.begin();
+             it != active_timeouts.end(); ++it)
+        {
+          if (it->second == timer)
+          {
+            active_timeouts.erase(it);
+            break;
+          }
+        }
+      }
+      [timer invalidate];
+      [self autorelease];
+    }
+  }
+  catch (std::exception &exc)
+  {
+    log_error("Unhandled exception calling timer callback: %s\n", exc.what());
   }
 }
 
 @end
 
-
-static void util_add_timeout(float interval, const boost::function<bool ()> &callback)
-{ 
-  MFTimerHandler *handler = [[MFTimerHandler alloc] initWithSlot:callback];
+static mforms::TimeoutHandle util_add_timeout(float interval, const boost::function<bool ()> &callback)
+{
+  base::MutexLock lock(timeout_lock);
   
-  [NSTimer scheduledTimerWithTimeInterval:interval
-                                   target:handler selector:@selector(fire:)
-                                 userInfo:nil repeats:YES];
+  MFTimerHandler *handler = [[MFTimerHandler alloc] initWithSlot:callback];
+
+  active_timeouts[++current_timeout] = [NSTimer scheduledTimerWithTimeInterval:interval
+                                                                        target:handler selector:@selector(fire:)
+                                                                      userInfo:nil repeats:YES];
+  return current_timeout;
 }
 
+
+static void util_cancel_timeout(mforms::TimeoutHandle handle)
+{
+  base::MutexLock lock(timeout_lock);
+  std::map<mforms::TimeoutHandle, NSTimer*>::iterator it;
+  
+  if ((it = active_timeouts.find(handle)) != active_timeouts.end())
+  {
+    [it->second invalidate];
+    active_timeouts.erase(it);
+  }
+  else
+    log_warning("cancel_timeout called on invalid handle %i\n", handle);
+}
+
+
 static std::string os_error_to_string(OSErr error)
 {
   CFStringRef ref = SecCopyErrorMessageString(error, nil);
@@ -255,8 +312,8 @@ static void util_forget_password(const std::string &service, const std::string &
 
 static void util_show_wait_message(const std::string &title, const std::string &message)
 {
-  [MHudController showHudWithTitle: [NSString stringWithUTF8String: title.c_str()]
-                    andDescription: [NSString stringWithUTF8String: message.c_str()]];
+  [MHudController showHudWithTitle: @(title.c_str())
+                    andDescription: @(message.c_str())];
 }
 
 
@@ -264,10 +321,9 @@ static bool util_run_cancelable_wait_message(const std::string &title, const std
                                              const boost::function<void ()> &start_task, 
                                              const boost::function<bool ()> &cancel_task)
 {
-  if (start_task)
-    start_task();
-  return [MHudController runModalHudWithTitle: [NSString stringWithUTF8String: title.c_str()]
-                               andDescription: [NSString stringWithUTF8String: text.c_str()]
+  return [MHudController runModalHudWithTitle: @(title.c_str())
+                               andDescription: @(text.c_str())
+                                  notifyReady: start_task
                                  cancelAction: cancel_task];
 }
 
@@ -355,12 +411,50 @@ static void *util_perform_from_main_thread(const boost::function<void* ()> &slot
   }
 }
 
+static void util_set_thread_name(const std::string &name)
+{
+  @autoreleasepool {
+    [[NSThread currentThread] setName: @(name.c_str())];
+  }
+}
+
+static double util_get_text_width(const std::string &text, const std::string &font_desc)
+{
+  static NSDictionary *attributeDict = nil;
+  static std::string cachedFontName;
+
+  if (!attributeDict || cachedFontName != font_desc)
+  {
+    std::string font;
+    float size;
+    bool bold, italic;
+    [attributeDict release];
+    attributeDict = nil;
+    if (base::parse_font_description(font_desc, font, size, bold, italic))
+    {
+      NSFontDescriptor *fd = [NSFontDescriptor fontDescriptorWithName: @(font.c_str()) size: size];
+      NSFont *font = [NSFont fontWithDescriptor: [fd fontDescriptorWithSymbolicTraits: (bold ? NSFontBoldTrait : 0) | (italic ? NSFontItalicTrait : 0)]
+                                      size: size];
+
+      attributeDict = [@{NSFontAttributeName: font} retain];
+    }
+    cachedFontName = font_desc;
+  }
+
+  NSAttributedString *str = [[NSAttributedString alloc] initWithString: @(text.c_str())
+                                                            attributes: attributeDict];
+  double w = [str size].width;
+  [str release];
+  return w;
+}
+
 //--------------------------------------------------------------------------------------------------
 
 void cf_util_init()
 {
   ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();
-    
+
+  f->_utilities_impl.beep = &util_beep;
   f->_utilities_impl.show_message= &util_show_message;
   f->_utilities_impl.show_error= &util_show_message;
   f->_utilities_impl.show_warning= &util_show_message;
@@ -373,6 +467,7 @@ void cf_util_init()
   f->_utilities_impl.get_clipboard_text= &util_get_clipboard_text;
   f->_utilities_impl.open_url= &util_open_url;
   f->_utilities_impl.add_timeout= &util_add_timeout;
+  f->_utilities_impl.cancel_timeout= &util_cancel_timeout;
   f->_utilities_impl.get_special_folder= &get_special_folder;
   f->_utilities_impl.store_password= &util_store_password;
   f->_utilities_impl.find_password= &util_find_password;
@@ -380,4 +475,6 @@ void cf_util_init()
   f->_utilities_impl.move_to_trash= &util_move_to_trash;
   f->_utilities_impl.reveal_file= &reveal_file;
   f->_utilities_impl.perform_from_main_thread= &util_perform_from_main_thread;
+  f->_utilities_impl.set_thread_name= &util_set_thread_name;
+  f->_utilities_impl.get_text_width = &util_get_text_width;
 }
