@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,14 +16,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
  * 02110-1301  USA
  */
-
-#include "stdafx.h"
-
+#include "grtpp_shell_python.h"
 #include "base/string_utilities.h"
 #include "base/threading.h"
 #include "base/log.h"
 
-#include "grtpp_shell_python.h"
 #include "grtpp.h"
 #include "grtpp_util.h"
 #include "grtpp_shell.h"
@@ -31,12 +28,8 @@
 #include "grtpp_undo_manager.h"
 #include "grtpp_notifications.h"
 
-#include <algorithm>
-
-#include "grtpp_shell_lua.h"
-
 #include <cppconn/exception.h>
-
+#include <algorithm>
 #include <glib.h>
 
 #include "serializer.h"
@@ -381,7 +374,7 @@ int GRT::scan_metaclasses_in(const std::string &directory, std::multimap<std::st
   
   g_dir_close(dir);
   
-  return _metaclasses.size() - old_count;
+  return (int)(_metaclasses.size() - old_count);
 }
 
 
@@ -670,10 +663,9 @@ bool GRT::load_module(const std::string &path, bool refresh)
     {
       log_debug2("Trying to load module '%s' (%s)\n", path.c_str(), (*loader)->get_loader_name().c_str());
       
-      Module *module= (*loader)->init_module(path);
-      if (!module)
-        log_error("Failed loading module '%s' (%s)\n", path.c_str(), (*loader)->get_loader_name().c_str());
-      else
+      // Problems, if any, are logged in init_module.
+      Module *module = (*loader)->init_module(path);
+      if (module)
       {
         try 
         {
@@ -971,12 +963,8 @@ void *GRT::get_context_data(const std::string &key)
 
 bool GRT::init_shell(const std::string &shell_type)
 {
-  if (shell_type == LanguageLua)
-    _shell= new LuaShell(this);
-#ifdef ENABLE_PYTHON_MODULES
-  else if (shell_type == LanguagePython)
+  if (shell_type == LanguagePython)
     _shell= new PythonShell(this);
-#endif
   else
     throw std::runtime_error("Invalid shell type "+shell_type);
   
@@ -987,12 +975,9 @@ bool GRT::init_shell(const std::string &shell_type)
 
 std::string GRT::shell_type()
 {
-  if (dynamic_cast<LuaShell*>(_shell))
-    return LanguageLua;
-#ifdef ENABLE_PYTHON_MODULES
-  else if (dynamic_cast<PythonShell*>(_shell))
+  if (dynamic_cast<PythonShell*>(_shell))
     return LanguagePython;
-#endif
+
   return "";
 }
 
@@ -1022,9 +1007,9 @@ void GRT::pop_message_handler()
 
 bool GRT::handle_message(const Message &msg, void *sender)
 {
-  if (_message_slot_stack.empty())
-    log_error("Unhandled message: %s\n", msg.format().c_str());
-  else
+  // Don't log any message if there's no message slot is occupied. It just means
+  // we don't want anything logged.
+  if (!_message_slot_stack.empty())
   {
     int i = 0;
     MessageSlot slot;
@@ -1044,7 +1029,7 @@ bool GRT::handle_message(const Message &msg, void *sender)
         return true;
     }
   }
-  log_error("Unhandled message (%lu): %s\n", _message_slot_stack.size(), msg.format().c_str());
+  log_error("Unhandled message (%lu): %s\n", (unsigned long)_message_slot_stack.size(), msg.format().c_str());
   return false;
 }
 
@@ -1109,7 +1094,7 @@ void GRT::send_warning(const std::string &message, const std::string &details, v
   msg.progress= 0.0;
   handle_message(msg, sender);
   
-  log_warning("%s\t%s", message.c_str(), details.c_str());
+  log_warning("%s\t%s\n", message.c_str(), details.c_str());
 }
 
 
@@ -1170,10 +1155,9 @@ void GRT::send_progress(float percentage, const std::string &message, const std:
   // calculate the actual progress percentage
   if (!_progress_step_stack.empty())
   {
-    for (int i = _progress_step_stack.size()-1; i >= 0; --i)
-    {
-      percentage = _progress_step_stack[i].first + (_progress_step_stack[i].second - _progress_step_stack[i].first) * percentage;
-    } 
+    std::vector<std::pair<float,float> >::reverse_iterator rit;
+    for (rit = _progress_step_stack.rbegin(); rit != _progress_step_stack.rend(); ++rit)
+      percentage = (*rit).first + ((*rit).second - (*rit).first) * percentage;
   }
   msg.progress= percentage;
   handle_message(msg, sender);
