@@ -29,13 +29,18 @@
 
 static void call_refresh(DbMysqlRelationshipEditor *self)
 {
-  [self performSelectorOnMainThread:@selector(refresh) withObject:nil waitUntilDone:YES];
+  [self performSelectorOnMainThread: @selector(refresh) withObject: nil waitUntilDone: YES];
 }
 
 
-- (id)initWithModule:(grt::Module*)module GRTManager:(bec::GRTManager*)grtm arguments:(const grt::BaseListRef&)args
+- (instancetype)initWithModule: (grt::Module*)module
+                    grtManager: (bec::GRTManager *)grtm
+                     arguments: (const grt::BaseListRef &)args
 {
-  self= [super initWithNibName: @"MySQLRelationshipEditor" bundle: [NSBundle bundleForClass:[self class]]];
+  if (grtm == nil)
+    return nil;
+
+  self = [super initWithNibName: @"MySQLRelationshipEditor" bundle: [NSBundle bundleForClass: [self class]]];
   if (self != nil)
   {
     _grtm = grtm;
@@ -47,7 +52,16 @@ static void call_refresh(DbMysqlRelationshipEditor *self)
   }
   return self;
 }
-    
+
+- (instancetype)initWithNibName: (NSString *)nibNameOrNil bundle: (NSBundle *)nibBundleOrNil
+{
+  return [self initWithModule: nil grtManager: nil arguments: grt::BaseListRef()];
+}
+
+-(instancetype)initWithCoder: (NSCoder *)coder
+{
+  return [self initWithModule: nil grtManager: nil arguments: grt::BaseListRef()];
+}
 
 - (void)reinitWithArguments: (const grt::BaseListRef&)args
 {
