@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,9 +35,11 @@ static void call_refresh(DbMysqlViewEditor *self)
   [self performSelectorOnMainThread:@selector(refresh) withObject:nil waitUntilDone:YES];
 }
 
-- (id)initWithModule:(grt::Module*)module GRTManager:(bec::GRTManager*)grtm arguments:(const grt::BaseListRef&)args
+- (instancetype)initWithModule: (grt::Module*)module
+                    grtManager: (bec::GRTManager *)grtm
+                     arguments: (const grt::BaseListRef &)args
 {
-  self= [super initWithNibName: @"MySQLViewEditor"  bundle: [NSBundle bundleForClass:[self class]]];
+  self = [super initWithNibName: @"MySQLViewEditor"  bundle: [NSBundle bundleForClass:[self class]]];
   if (self != nil)
   {
     _grtm = grtm;
