@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2010, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,10 +19,18 @@
 
 #pragma once
 
+using namespace System;
+
 #define NOMINMAX
 #define WIN32_LEAN_AND_MEAN 
 #include <windows.h>
 
-#pragma warning(disable: 4793)  // 'vararg' causes native code generation
-
 #include <algorithm>
+
+#include <glib.h>
+
+#include "grts/structs.db.mgmt.h"
+#include "grts/structs.db.mysql.h"
+#include "grts/structs.db.query.h"
+#include "grts/structs.model.h"
+#include "grts/structs.workbench.physical.h"
