@@ -96,6 +96,15 @@ class MigrationTarget(object):
     def module_db(self):
         return grt.modules.DbMySQL
 
+    def get_os(self):
+        if self.checkConnection():
+            os_name = self.module_fe().getOS(self.connection)
+            self.disconnect()
+            return os_name
+
+        return None
+
+
 
 class MigrationSource(object):
     def __init__(self, state, connection):
@@ -190,9 +199,9 @@ class MigrationSource(object):
             raise NotSupportedError('MySQL connections through SSH are not supported in this version '
                                     'of the MySQL Workbench Migration Wizard.')
         self._rev_eng_module.connect(self.connection, self.password or "")
-        if str(self.getDriverDBMSName()).upper() == 'ACCESS':
-            raise NotSupportedError('Microsoft Access is not supported in this version of the MySQL Workbench '
-                                    'Migration Wizard.')
+        #if str(self.getDriverDBMSName()).upper() == 'ACCESS':
+        #    raise NotSupportedError('Microsoft Access is not supported in this version of the MySQL Workbench '
+        #                            'Migration Wizard.')
         return True
 
     def disconnect(self):
@@ -235,26 +244,30 @@ class MigrationSource(object):
         
         grt.send_progress(0.0, "Checking connection...")
         self.connect()
-        if self.rdbms.doesSupportCatalogs:
+        if self.rdbms.doesSupportCatalogs > 0:
             grt.send_progress(0.1, "Fetching catalog names...")
             self.state.sourceSchemataNames.remove_all()
             catalog_names = self.getCatalogNames()
             if only_these_catalogs:
                 inexistent_catalogs = set(only_these_catalogs).difference(catalog_names)
                 if inexistent_catalogs:
-                    grt.send_warning('The following catalogs where not found: ' + ', '.join(list(inexistent_catalogs)))
+                    grt.send_warning('The following catalogs were not found: ' + ', '.join(list(inexistent_catalogs)))
                 catalog_names = list(set(only_these_catalogs).difference(inexistent_catalogs)) or self.getCatalogNames()
             self._catalog_names = catalog_names
             grt.send_progress(0.1, "Fetching schema names...")
             accumulated_progress = 0.1
             step_progress_share = 1.0 / (len(catalog_names) + 1e-10)
             for catalog in catalog_names:
+                if not catalog:
+                    continue
                 grt.send_progress(accumulated_progress, 'Fetching schema names from %s...' % catalog)
                 schema_names = self.getSchemaNames(catalog)
                 for schema in schema_names:
+                    if not schema:
+                        continue
                     self.state.sourceSchemataNames.append("%s.%s" % (self._db_module.quoteIdentifier(catalog), self._db_module.quoteIdentifier(schema)))
                 accumulated_progress += 0.9 * step_progress_share
-        else:  # The rdbms doesn't support catalogs
+        elif self.rdbms.doesSupportCatalogs == 0:  # The rdbms doesn't support catalogs
             grt.send_progress(0.1, "Fetching schema names...")
             schema_names = self.getSchemaNames('')
             if only_these_catalogs:  # Here only_these_catalogs would rather mean only these schemata
@@ -266,6 +279,11 @@ class MigrationSource(object):
             self.state.sourceSchemataNames.remove_all()
             for schema in schema_names:
                 self.state.sourceSchemataNames.append('%s.%s' % (self._db_module.quoteIdentifier('def'), self._db_module.quoteIdentifier(schema)))
+        else: # no schema either
+            self._catalog_names = []
+            self.state.sourceSchemataNames.remove_all()
+            for schema in self.getSchemaNames(''):
+                self.state.sourceSchemataNames.append(self._db_module.quoteIdentifier(schema))
         grt.send_progress(1.0, "Finished")
 
 
@@ -358,6 +376,12 @@ class MigrationSource(object):
         if hasattr(self._rev_eng_module, "cleanup"):
             self._rev_eng_module.cleanup()
 
+    def get_os(self):
+        return self.module_re().getOS(self.connection)
+
+    def get_source_instance(self):
+        return self.module_re().getSourceInstance(self.connection)
+
 
 
 class MigrationPlan(object):
@@ -388,11 +412,11 @@ class MigrationPlan(object):
         elif sys.platform == "darwin":
             self.wbcopytables_path = mforms.App.get().get_executable_path("wbcopytables")
         else:
-            self.wbcopytables_path = mforms.App.get().get_executable_path("wbcopytables")
+            self.wbcopytables_path = mforms.App.get().get_executable_path("wbcopytables-bin")
             if not os.path.exists(self.wbcopytables_path):
-                self.wbcopytables_path = os.path.join(os.path.dirname(grt.root.wb.registry.appExecutablePath), "wbcopytables")
+                self.wbcopytables_path = os.path.join(os.path.dirname(grt.root.wb.registry.appExecutablePath), "wbcopytables-bin")
             if not os.path.exists(self.wbcopytables_path):
-                self.wbcopytables_path = "wbcopytables"
+                self.wbcopytables_path = "wbcopytables-bin"
         if type(self.wbcopytables_path) == unicode:
             self.wbcopytables_path = self.wbcopytables_path.encode("UTF-8")
 
