@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2014 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012, 2014, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,7 +17,13 @@
  * 02110-1301  USA
  */
 
+#ifdef _WIN32
+#define HAVE_ROUND
+#endif
+
 #include "python_copy_data_source.h" // python stuff need to be 1st #include
+#include "copytable.h"
+
 #include <cstdlib>
 #include <cstdio>
 #include <iostream>
@@ -29,18 +35,14 @@
 
 #include "boost/scoped_ptr.hpp"
 
-#include "copytable.h"
 #undef tolower
 #undef toupper
 
 #include "base/string_utilities.h"
 #include "base/file_utilities.h"
 
-#include "wb_version.h"
+#include "workbench/wb_version.h"
 
-#if defined(WIN32)
-#define atoll _atoi64
-#endif
 
 class input_error : public std::runtime_error
 {
@@ -52,10 +54,11 @@ public:
 DEFAULT_LOG_DOMAIN("copytable");
 
 
-static void count_rows(boost::scoped_ptr<CopyDataSource>& source, const std::string &source_schema, const std::string &source_table,
-                       const CopySpec &spec)
+static void count_rows(boost::scoped_ptr<CopyDataSource>& source, const std::string &source_schema,
+                       const std::string &source_table, const std::vector<std::string> &pk_columns,
+                       const CopySpec &spec, const std::vector<std::string> &last_pkeys)
 {
-  unsigned long long total = source->count_rows(source_schema, source_table, spec);
+  unsigned long long total = source->count_rows(source_schema, source_table, pk_columns, spec, last_pkeys);
 
   printf("ROW_COUNT:%s:%s: %llu\n", source_schema.c_str(), source_table.c_str(), total);
   fflush(stdout);
@@ -160,12 +163,14 @@ static void show_help()
   printf("--count-only\n");
   printf("--jobs-from-stdin\n");
   printf("--abort-on-oversized-blobs\n");
+  printf("--max-count=<max rows count>\n");
+  printf("--resume\n");
   printf("Table Specification from file:\n");
   printf("--table-file=<filename>\n");
-  printf("<source schema><TAB><source table><TAB><target schema><TAB><target table><TAB>*|<select expression>\n");
+  printf("<source schema><TAB><source table><TAB><target schema><TAB><target table><TAB><source pk columns><TAB><target pk columns><TAB>*|<select expression>\n");
   printf("Table Specification from command line:\n");
-  printf("--table <source schema> <source table> <target schema> <target table> *|<select expression>\n");
-  printf("--table-range <source schema> <source table> <target schema> <target table> <source key> <start>|-1 <end>|-1\n");
+  printf("--table <source schema> <source table> <target schema> <target table> <source pk columns> <target pk columns> *|<select expression>\n");
+  printf("--table-where <source schema> <source table> <target schema> <target table> <source pk columns> <target pk columns> *|<select expression> <where expression>\n");
   printf("\n");
   printf("--log-file=<file_path>\n");
   printf("--log-level=<level>\n");
@@ -186,17 +191,21 @@ static void show_help()
 *                from the source DB or to actually trasmit the data
 * - tasks : output parameter that will contain a task for each table definition loaded
 *           from the file
+  - resume : indicates if the file contains information to resume copying data from
+             last PK
+* - max_count : limit copied rows count to max_count
 *
-* Remarks : Each table is defined in a single line with the next format for count_only = true
+* Remarks : Each table is defined in a single line with the next format for count_only = true and resume = false
 *           <src_schema>\t<src_table>\n
 *
-*           and in the next format for a count_only = false
-*           <src_schema>\t<src_table>\t<tgt_schema>\t<tgt_table>\t<select_expression>
+*           and in the next format for a count_only = false or resume = true
+*           <src_schema>\t<src_table>\t<tgt_schema>\t<tgt_table>\t<source_pk_columns>\t<target_pk_columns>\t<select_expression>
 */
-bool read_tasks_from_file(const std::string file_name, bool count_only, TaskQueue& tasks, std::set<std::string> &trigger_schemas)
+bool read_tasks_from_file(const std::string file_name, bool count_only, TaskQueue& tasks, std::set<std::string> &trigger_schemas,
+                          bool resume, long long int max_count)
 {
   std::ifstream ifs ( file_name.data() , std::ifstream::in );
-  unsigned int field_count = count_only ? 2 : 5;
+  unsigned int field_count = count_only && !resume ? 2 : 7;
   bool error = false;
 
   printf("Loading table information from file %s\n", file_name.data());
@@ -218,15 +227,19 @@ bool read_tasks_from_file(const std::string file_name, bool count_only, TaskQueu
         param.source_schema = fields[0];
         param.source_table = fields[1];
 
-        if (!count_only)
+        if (!(count_only && !resume))
         {
           param.target_schema = fields[2];
           param.target_table = fields[3];
-          param.select_expression = fields[4];
+          param.source_pk_columns = base::split(fields[4], ",", -1);
+          param.target_pk_columns = base::split(fields[5], ",", -1);
+          param.select_expression = fields[6];
 
           trigger_schemas.insert(param.target_schema);
         }
 
+        param.copy_spec.resume = resume;
+        param.copy_spec.max_count = max_count;
         param.copy_spec.type = CopyAll;
         tasks.add_task(param);
       }
@@ -250,11 +263,14 @@ int main(int argc, char **argv)
 
   std::string source_password;
   std::string source_connstring;
+  bool source_use_cleartext_plugin = false;
   bool source_is_utf8 = false;
+  std::string source_charset;
   SourceType source_type = ST_MYSQL;
 
   std::string target_connstring;
   std::string target_password;
+  bool target_use_cleartext_plugin = false;
   std::string log_level;
   std::string log_file;
 
@@ -267,12 +283,15 @@ int main(int argc, char **argv)
   bool disable_triggers = false;
   bool reenable_triggers = false;
   bool disable_triggers_on_copy = true;
+  bool resume = false;
   int thread_count = 1;
-  int bulk_insert_batch = 100;
+  long long bulk_insert_batch = 100;
+  long long max_count = 0;
 
   std::string table_file;
 
   std::set<std::string> trigger_schemas;
+  std::string source_rdbms_type = "unknown";
 
   bool log_level_set = false;
   int i = 1;
@@ -305,6 +324,8 @@ int main(int argc, char **argv)
       target_password = argval;
     else if (strcmp(argv[i], "--force-utf8-for-source") == 0)
       source_is_utf8 = true;
+    else if (check_arg_with_value(argv, i, "--source-charset", argval, true))
+      source_charset = argval;
     else if (strcmp(argv[i], "--progress") == 0)
       show_progress = true;
     else if (strcmp(argv[i], "--truncate-target") == 0)
@@ -324,6 +345,8 @@ int main(int argc, char **argv)
       abort_on_oversized_blobs = true;
     else if (strcmp(argv[i], "--dont-disable-triggers") == 0)
       disable_triggers_on_copy = false;
+    else if (strcmp(argv[i], "--resume") == 0)
+      resume = true;
     else if (check_arg_with_value(argv, i, "--disable-triggers-on", argval, true))
     {
       // disabling/enabling triggers are standalone operations and mutually exclusive
@@ -346,13 +369,13 @@ int main(int argc, char **argv)
     }
     else if (check_arg_with_value(argv, i, "--thread-count", argval, true))
     {
-      thread_count = atoi(argval);
+      thread_count = base::atoi<int>(argval, 0);
       if (thread_count < 1)
         thread_count = 1;
     }
     else if (check_arg_with_value(argv, i, "--bulk-insert-batch-size", argval, true))
     {
-      bulk_insert_batch = atoi(argval);
+      bulk_insert_batch = base::atoi<int>(argval, 0);
       if (bulk_insert_batch < 1)
         bulk_insert_batch = 100;
     }
@@ -389,21 +412,28 @@ int main(int argc, char **argv)
     {
       TableParam param;
 
-      if ((!count_only && i + 5 >= argc) || (count_only && i + 2 >= argc))
+      if ((!count_only && i + 7 >= argc) || (count_only && i + 2 >= argc))
       {
         fprintf(stderr, "%s: Missing value for table copy specification\n", argv[0]);
         exit(1);
       }
+
       param.source_schema = argv[++i];
       param.source_table = argv[++i];
-      if (!count_only)
+      if (!(count_only && !resume))
       {
         param.target_schema = argv[++i];
         param.target_table = argv[++i];
+        param.source_pk_columns = base::split(argv[++i], ",", -1);
+        param.target_pk_columns = base::split(argv[++i], ",", -1);
         param.select_expression = argv[++i];
 
         trigger_schemas.insert(param.target_schema);
       }
+
+      param.copy_spec.resume = resume;
+      param.copy_spec.max_count = max_count;
+
       param.copy_spec.type = CopyAll;
 
       tables.add_task(param);
@@ -412,23 +442,26 @@ int main(int argc, char **argv)
     {
       TableParam param;
 
-      if ((!count_only && i + 7 >= argc) || (count_only && i + 5 >= argc))
+      if ((!count_only && i + 10 >= argc) || (count_only && i + 5 >= argc))
       {
         fprintf(stderr, "%s: Missing value for table copy specification\n", argv[0]);
         exit(1);
       }
       param.source_schema = argv[++i];
       param.source_table = argv[++i];
-      if (!count_only)
+      if (!(count_only && !resume))
       {
         param.target_schema = argv[++i];
         param.target_table = argv[++i];
+        param.source_pk_columns = base::split(argv[++i], ",", -1);
+        param.target_pk_columns = base::split(argv[++i], ",", -1);
+        param.select_expression = argv[++i];
 
         trigger_schemas.insert(param.target_schema);
       }
       param.copy_spec.range_key = argv[++i];
-      param.copy_spec.range_start = atoll(argv[++i]);
-      param.copy_spec.range_end = atoll(argv[++i]);
+      param.copy_spec.range_start = base::atoi<long long>(argv[++i], 0ll);
+      param.copy_spec.range_end = base::atoi<long long>(argv[++i], 0ll);
       param.copy_spec.type = CopyRange;
 
       tables.add_task(param);
@@ -437,23 +470,68 @@ int main(int argc, char **argv)
     {
       TableParam param;
 
-      if ((!count_only && i + 5 >= argc) || (count_only && i + 3 >= argc))
+      if ((!count_only && i + 8 >= argc) || (count_only && i + 3 >= argc))
       {
         fprintf(stderr, "%s: Missing value for table copy specification\n", argv[0]);
         exit(1);
       }
       param.source_schema = argv[++i];
       param.source_table = argv[++i];
-      if (!count_only)
+      if (!(count_only && !resume))
       {
         param.target_schema = argv[++i];
         param.target_table = argv[++i];
+        param.source_pk_columns = base::split(argv[++i], ",", -1);
+        param.target_pk_columns = base::split(argv[++i], ",", -1);
+        param.select_expression = argv[++i];
       }
-      param.copy_spec.row_count = atoll(argv[++i]);
+      param.copy_spec.row_count = base::atoi<long long>(argv[++i], 0ll);
+      param.copy_spec.resume = resume;
       param.copy_spec.type = CopyCount;
 
       tables.add_task(param);
     }
+    else if (check_arg_with_value(argv, i, "--source-rdbms-type", argval, false))
+    	source_rdbms_type = argval;
+    else if (strcmp(argv[i], "--table-where") == 0)
+    {
+      TableParam param;
+
+      if ((!count_only && i + 8 >= argc) || (count_only && i + 4 >= argc))
+      {
+        fprintf(stderr, "%s: Missing value for table copy specification\n", argv[0]);
+        exit(1);
+      }
+      param.source_schema = argv[++i];
+      param.source_table = argv[++i];
+      if (!(count_only && !resume))
+      {
+        param.target_schema = argv[++i];
+        param.target_table = argv[++i];
+        param.source_pk_columns = base::split(argv[++i], ",", -1);
+        param.target_pk_columns = base::split(argv[++i], ",", -1);
+        param.select_expression = argv[++i];
+        param.copy_spec.where_expression = argv[++i];
+
+        trigger_schemas.insert(param.target_schema);
+      }
+      else
+      {
+        param.select_expression = argv[++i];
+        param.copy_spec.where_expression = argv[++i];
+      }
+      param.copy_spec.type = CopyWhere;
+
+      tables.add_task(param);
+    }
+    else if (check_arg_with_value(argv, i, "--max-count", argval, true))
+    {
+      max_count = base::atoi<int>(argval, 0);
+    }
+    else if (strcmp(argv[i], "--source-use-cleartext") == 0)
+      source_use_cleartext_plugin = true;
+    else if (strcmp(argv[i], "--target-use-cleartext") == 0)
+      target_use_cleartext_plugin = true;
     else
     {
       fprintf(stderr, "%s: Invalid option %s\n", argv[0], argv[i]);
@@ -494,9 +572,9 @@ int main(int argc, char **argv)
   // If needed, reads the tasks from the table definition file
   if (!table_file.empty())
   {
-    if (!read_tasks_from_file(table_file, count_only, tables, trigger_schemas))
+    if (!read_tasks_from_file(table_file, count_only, tables, trigger_schemas, resume, max_count))
     {
-      fprintf(stderr, "Error reading table definitions from table file: %s\n", table_file.data());
+      fprintf(stderr, "Invalid table definitions format in file: %s\n", table_file.data());
       exit(1);
     }
   }
@@ -510,7 +588,7 @@ int main(int argc, char **argv)
     exit(1);
   }
 
-  if (target_connstring.empty() && !count_only)
+  if (target_connstring.empty() && !(count_only && !resume))
   {
     fprintf(stderr, "Missing target DB server\n");
     exit(1);
@@ -545,7 +623,7 @@ int main(int argc, char **argv)
   std::string target_user;
   int target_port = -1;
   std::string target_socket;
-  if (!count_only && !parse_mysql_connstring(target_connstring, target_user, target_password,
+  if (!(count_only && !resume) && !parse_mysql_connstring(target_connstring, target_user, target_password,
                                              target_host, target_port, target_socket))
   {
     fprintf(stderr, "Invalid MySQL connection string %s for target database. Must be in format user[:pass]@host:port or user[:pass]@::socket\n", target_connstring.c_str());
@@ -561,7 +639,7 @@ int main(int argc, char **argv)
       exit(1);
     }
 
-    if (count_only || reenable_triggers || disable_triggers)
+    if ((count_only && !resume)|| reenable_triggers || disable_triggers)
     {
       char *ptr = strtok(password, "\t\r\n");
       if (ptr)
@@ -609,21 +687,31 @@ int main(int argc, char **argv)
         SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &odbc_env);
         SQLSetEnvAttr(odbc_env, SQL_ATTR_ODBC_VERSION, (void *) SQL_OV_ODBC3, 0);
 
-        psource.reset(new ODBCCopyDataSource(odbc_env, source_connstring, source_password, source_is_utf8));
+        psource.reset(new ODBCCopyDataSource(odbc_env, source_connstring, source_password, source_is_utf8, source_rdbms_type));
       }
       else if (source_type == ST_MYSQL)
-        psource.reset(new MySQLCopyDataSource(source_host, source_port, source_user, source_password, source_socket));
+        psource.reset(new MySQLCopyDataSource(source_host, source_port, source_user, source_password, source_socket, source_use_cleartext_plugin));
       else
         psource.reset(new PythonCopyDataSource(source_connstring, source_password));
 
+      boost::scoped_ptr<MySQLCopyDataTarget> ptarget;
       TableParam task;
       while(tables.get_task(task))
-        count_rows(psource, task.source_schema, task.source_table, task.copy_spec);
+      {
+        std::vector<std::string> last_pkeys;
+        if (task.copy_spec.resume)
+        {
+          if(!ptarget.get())
+            ptarget.reset(new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, target_use_cleartext_plugin, app_name, source_charset, source_rdbms_type));
+          last_pkeys = ptarget->get_last_pkeys(task.target_pk_columns, task.target_schema, task.target_table);
+        }
+        count_rows(psource, task.source_schema, task.source_table, task.source_pk_columns, task.copy_spec, last_pkeys);
+      }
     }
     else if (reenable_triggers || disable_triggers)
     {
       boost::scoped_ptr<MySQLCopyDataTarget> ptarget;
-      ptarget.reset(new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, app_name));
+      ptarget.reset(new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, target_use_cleartext_plugin, app_name, source_charset, source_rdbms_type));
 
       if (disable_triggers)
         ptarget->backup_triggers(trigger_schemas);
@@ -640,7 +728,7 @@ int main(int argc, char **argv)
 
       if (disable_triggers_on_copy)
       {
-        ptarget_conn.reset(new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, app_name));
+        ptarget_conn.reset(new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, target_use_cleartext_plugin, app_name, source_charset, source_rdbms_type));
         ptarget_conn->backup_triggers(trigger_schemas);
       }
 
@@ -651,24 +739,27 @@ int main(int argc, char **argv)
           SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE, &odbc_env);
           SQLSetEnvAttr(odbc_env, SQL_ATTR_ODBC_VERSION, (void *) SQL_OV_ODBC3, 0);
 
-          psource = new ODBCCopyDataSource(odbc_env, source_connstring, source_password, source_is_utf8);
+          psource = new ODBCCopyDataSource(odbc_env, source_connstring, source_password, source_is_utf8, source_rdbms_type);
         }
         else if (source_type == ST_MYSQL)
-          psource = new MySQLCopyDataSource(source_host, source_port, source_user, source_password, source_socket);
+          psource = new MySQLCopyDataSource(source_host, source_port, source_user, source_password, source_socket, source_use_cleartext_plugin);
         else
           psource = new PythonCopyDataSource(source_connstring, source_password);
 
-        ptarget = new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, app_name);
+        ptarget = new MySQLCopyDataTarget(target_host, target_port, target_user, target_password, target_socket, target_use_cleartext_plugin, app_name, source_charset, source_rdbms_type);
 
         psource->set_max_blob_chunk_size(ptarget->get_max_allowed_packet());
-        psource->set_max_parameter_size(ptarget->get_max_long_data_size());
+        psource->set_max_parameter_size((unsigned long)ptarget->get_max_long_data_size());
         psource->set_abort_on_oversized_blobs(abort_on_oversized_blobs);
         ptarget->set_truncate(truncate_target);
-        ptarget->set_bulk_insert_batch_size(bulk_insert_batch);
+        if (max_count > 0)
+          bulk_insert_batch = max_count;
+        ptarget->set_bulk_insert_batch_size((int)bulk_insert_batch);
 
         if (check_types_only)
         {
           //XXXX
+          delete psource;
         }
         else
         {
