@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -51,7 +51,8 @@ class WbAdminConfigurationStartup(mforms.Box):
     #---------------------------------------------------------------------------
     def print_output(self, text):
         ts = datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S - ")
-        self.startup_msgs_log.append_text_with_encoding(ts + text + "\n", self.ctrl_be.server_helper.cmd_output_encoding)
+        if self.startup_msgs_log:
+            self.startup_msgs_log.append_text_with_encoding(ts + text + "\n", self.ctrl_be.server_helper.cmd_output_encoding)
 
     #---------------------------------------------------------------------------
     def __init__(self, ctrl_be, server_profile, main_view):
@@ -158,9 +159,14 @@ class WbAdminConfigurationStartup(mforms.Box):
 
     #---------------------------------------------------------------------------
     def page_activated(self):
+        if not self.server_profile.admin_enabled:
+            self.add(no_remote_admin_warning_label(self.server_profile), True, True)
+            return
+        
         if not self.ui_created:
             self.create_ui()
             self.ui_created = True
+
         if self.server_control:
             self.server_control.set_output_handler(self.print_output_cb)
 
