@@ -1,4 +1,4 @@
-# Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2007, 2015, Oracle and/or its affiliates. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -286,10 +286,7 @@ class WbAdminServerStatus(mforms.Box):
         repl_error = None
         res = None
         try:
-            if self.ctrl_be.target_version and self.ctrl_be.target_version.is_supported_mysql_version_at_least(5, 7, 0):
-                res = self.ctrl_be.exec_query("SHOW SLAVE STATUS NONBLOCKING")
-            else:
-                res = self.ctrl_be.exec_query("SHOW SLAVE STATUS")
+            res = self.ctrl_be.exec_query("SHOW SLAVE STATUS")
         except QueryError, e:
             if e.error == 1227:
                 repl_error = "Insufficient privileges to view slave status"
@@ -324,7 +321,8 @@ class WbAdminServerStatus(mforms.Box):
         for key, (control, value_source) in self.controls.items():
             if callable(value_source):
                 if isinstance(control, mforms.Label):
-                    control.set_text(value_source(info, plugins))
+                    resp = value_source(info, plugins)
+                    control.set_text(resp if resp else "n/a")
                 else:
                     value = value_source(info, plugins)
                     if type(value) is tuple:
